#include "atlas_conf_2015_062.h"
#include "TMatrixD.h"
#include "TMatrixDEigen.h"
// AUTHOR: Krzysztof Rolbiecki
//  EMAIL: krolb@fuw.edu.pl
void Atlas_conf_2015_062::initialize() {
  setAnalysisName("atlas_conf_2015_062");          
  setInformation(""
    "# search for squarks and gluinos\n"
     "# 0 leptons, 2-6 jets, MET\n"
     "# 13 TeV, 3.2/fb\n"
  "");
  setLuminosity(3.2*units::INVFB);      
  ignore("towers"); // These won't read tower or track information from the
  ignore("tracks"); //  Delphes output branches to save computing time.
  bookSignalRegions("SR2jl;SR2jm;SR2jt;SR4jt;SR5j;SR6jm;SR6jt;");
  // You can also book cutflow regions with bookCutFlowRegions("CR1;CR2;..."). Note that the regions are
  //  always ordered alphabetically in the cutflow output files.

  // You should initialize any declared variables here
}

void Atlas_conf_2015_062::analyze() {
  // Your eventwise analysis code goes here
  // The following objects are always defined unless they are 'ignored' above. They form std::vector objects of the respective Delphes class type (except for Etmiss which is a single object)
  // All std::vector members and etmiss have the common properties PT, Eta, Phi and P4() with the latter giving access to the full ROOT TLorentzVector.
  // Within a std::vector, all members are ordered with highest pt coming first.

  // electronsLoose, electronsMedium, electronsTight   are list of electrons that passed respective efficiency and reconstruction cuts
  // muonsCombinedPlus, muonsCombined                  as above for muons
  // photons                                           as above for photons
  // jets are all reconstructed jets                   as above for jets. Note that electrons are most certainly also reconstructed as a jet -> overlap removal do avoid double counting necessary!
  // tracks, towers                                    calorimeter and tracker information. Usually not needed.
  // missingET                                         rec missing ET EXCLUDING muons.

  
  // Here is a couple of useful functions and lines:  
  //------------Phase Space Cuts (defined for jets, electronsXYZ, muonsXYZ, photonsXYZ)
  // jets = filterPhaseSpace(jets, 20., -2.8, 2.8)  // The vector 'jets' only contains jets with pt >= 20 GeV and -2.8 < eta < 2.8. This function is applicable to other particles too (electronsMedium, ... ).
  // jets = overlapRemoval(jets, electronsLoose, 0.2) Removes all jets for which there exists any electron in 'electronsLoose' with deltaR < 0.2.
  // jets = overlapRemoval(jets, 0.2) If two jets overlap within deltaR < 0.2, only the harder jet is stored.
  
  //------------Isolation Checks (defined for electronsXYZ, muonsXYZ, photonsXYZ
  //------------        For each object, if the user entered N isolation conditions, they can be
  //------------        checked individually be the second argument (running from 0 to N-1).
  // electronsMedium = filterIsolation(electronsMedium, 0)            Removes electrons that do not pass the first isolation condition entered into the AnalysisManager by the user
  // std::vector<int> flags; flags.push_back(0); flags.push_back(2);
  // electronsMedium = filterIsolation(electronsMedium, flags)        Same as above, but both the first and the third condition have to be fulfilled
  // electronsMedium = filterIsolation(electronsMedium)               Same as above, but all conditions have to be fulfilled.
  
  //-----------Flavour Tag Checks (defined for jets only)
  //----------          Tau tags "loose", "medium" or "tight" can be individually checked if the user enabled tau tagging in the AM.
  //----------          For b-tags, if N working points have been defined, the ith condition can be tested by putting i-1 into the second argument (if there is only one, the argument can be omitted)
  // if checkTauTag(jets[0], "tight") leadingJetIsTagged = True;
  // if checkBTag(jets[0], 0) leadingJetIsBTagged = True;


  //-----------Auxiliary Information
  // - Always ensure that you don't access vectors out of bounds. E.g. 'if(jets[1]->PT > 150)' should rather be if (jets.size() > 1 && jets[1]->PT > 150). 
  // - Use rand()/(RAND_MAX+1.) for random numbers between 0 and 1. The random seed is determined from system time or by the RandomSeed parameter in ChekćkMATE.
  // - The 'return' statement will end this function for the current event and hence should be called whenever the current event is to be vetoed.
  // - Many advanced kinematical functions like mT2 are implemented. Check the manual for more information.
  // - If you need output to be stored in other files than the cutflow/signal files we provide, check the manual for how to do this conveniently.  

  missingET->addMuons(muonsCombined);  // Adds muons to missing ET. This should almost always be done which is why this line is not commented out.

  countCutflowEvent("00_all");  
  
  jets = filterPhaseSpace(jets, 20., -2.8, 2.8);

  std::vector<Electron*> electronsBaseline = filterPhaseSpace(electronsMedium, 10., -2.47, 2.47);
  std::vector<Muon*> muonsBaseline         = filterPhaseSpace(muonsCombined, 10., -2.5, 2.5);

  jets = overlapRemoval(jets, electronsBaseline, 0.2);
  electronsBaseline = overlapRemoval(electronsBaseline, jets, 0.4);
  muonsBaseline     = overlapRemoval(muonsBaseline, jets, 0.4);
  
  //for veto where baseline leptons exist
  if ( electronsBaseline.size() + muonsBaseline.size() != 0 ) return;
  countCutflowEvent("01_leptonveto");
  
  //for veto where missingET < 200
  if ( missingET->P4().Et() < 200. ) return;
  countCutflowEvent("02_MET>200");  
  
  //for veto where pt(j1) < 200 
  if ( jets.size() < 2 || jets[0]->PT < 200. ) return;
  countCutflowEvent("03_ptj1>200");  
  
  bool flag_2jl, flag_2jm, flag_2jt, flag_4jt, flag_5j, flag_6jm, flag_6jt;
  flag_2jl = flag_2jm = flag_2jt = flag_4jt = flag_5j = flag_6jm = flag_6jt = false;
  
// first clasify according to the number of jets  
  
  if ( jets[0]->PT > 300. && jets[1]->PT > 50. ) {
    flag_2jm = true;
    countCutflowEvent("2jm_04_ptjets1&2");
  }
  
  if ( jets[1]->PT > 200. ) {
    flag_2jl = true;
    flag_2jt = true;
    countCutflowEvent("2jl_04_jet2>200");
    countCutflowEvent("2jt_04_jet2>200");
  }  
  
  if ( jets.size() > 3 && jets[3]->PT > 100. ) {
    flag_4jt = true;
    countCutflowEvent("4jt_04_jet4>100");
  }    
  
  if ( jets.size() > 4 && jets[4]->PT > 100. ) {
    flag_5j = true;
    countCutflowEvent("5j_04_jet5>100");
  }  
  
  if ( jets.size() > 5 && jets[5]->PT > 100. ) {
    flag_6jm = flag_6jt = true;
    countCutflowEvent("6jm_04_jet6>100");
    countCutflowEvent("6jt_04_jet6>100");
  }    
  
  if ( !(flag_2jl || flag_2jm || flag_2jt || flag_4jt || flag_5j || flag_6jm || flag_6jt) ) return;

// check separation of jets and MET  
  
  if ( flag_2jl || flag_2jt) {
    for (int i = 0; i < 2; i++ ) 
      if ( fabs( jets[i]->P4().DeltaPhi( missingET->P4())) < 0.8 ) 
	flag_2jl = flag_2jt = false;
    if (jets.size() > 2 && jets[2]->PT > 50. && fabs(jets[2]->P4().DeltaPhi( missingET->P4())) < 0.8 ) 
	flag_2jl = flag_2jt = false;  
  }
  if ( flag_2jl ) countCutflowEvent("2jl_05_dphi>0.8");   
  if ( flag_2jt ) countCutflowEvent("2jt_05_dphi>0.8");   
      
  if ( flag_2jm ) {
    for (int i = 0; i < 2; i++ ) 
      if ( fabs( jets[i]->P4().DeltaPhi( missingET->P4())) < 0.4 ) 
	flag_2jm  = false;
    if (jets.size() > 2 && jets[2]->PT > 50. && fabs(jets[2]->P4().DeltaPhi( missingET->P4())) < 0.4 ) 
	flag_2jm = false;        
  }
  if ( flag_2jm ) countCutflowEvent("2jm_05_dphi>0.4");     
  
  if ( flag_4jt ) {
    for (int i = 0; i < 3; i++ ) 
      if ( fabs( jets[i]->P4().DeltaPhi( missingET->P4())) < 0.4 ) 
	flag_4jt  = false;
    for (int i = 3; i < jets.size() && jets[i]->PT > 50.; i++ ) 
      if ( fabs( jets[i]->P4().DeltaPhi( missingET->P4())) < 0.2 ) 
	flag_4jt  = false;      
  }
  if ( flag_4jt  ) countCutflowEvent("4jt_05_dphi>0.4(0.2)"); 
  
  if ( flag_5j ) {
    for (int i = 0; i < 3; i++ ) 
      if ( fabs( jets[i]->P4().DeltaPhi( missingET->P4())) < 0.4 ) 
	flag_5j  = false;
    for (int i = 3; i < jets.size() && jets[i]->PT > 50.; i++ ) 
      if ( fabs( jets[i]->P4().DeltaPhi( missingET->P4())) < 0.2 ) 
	flag_5j  = false;      
  }
  if ( flag_5j ) countCutflowEvent("5j_05_dphi>0.4(0.2)");      
  
  if ( flag_6jm || flag_6jt  ) {
    for (int i = 0; i < 3; i++ ) 
      if ( fabs( jets[i]->P4().DeltaPhi( missingET->P4())) < 0.4 ) 
	flag_6jm = flag_6jt  = false;
    for (int i = 3; i < jets.size() && jets[i]->PT > 50.; i++ ) 
      if ( fabs( jets[i]->P4().DeltaPhi( missingET->P4())) < 0.2 ) 
	flag_6jm = flag_6jt  = false;
  }
  if ( flag_6jm ) countCutflowEvent("6jm_05_dphi>0.4(0.2)");     
  if ( flag_6jt ) countCutflowEvent("6jt_05_dphi>0.4(0.2)");     
  
  if ( !(flag_2jl || flag_2jm || flag_2jt || flag_4jt || flag_5j || flag_6jm || flag_6jt) ) return;  
  
// for 2 jets SRs check MET/sqrt(HT)    
  
  double ht = 0.;
  for (int i  = 0; i < jets.size() && jets[i]->PT > 50.; i++ )
    ht += jets[i]->PT; 
  
  if ( (flag_2jl || flag_2jm) && missingET->P4().Et()/sqrt(ht) < 15. )
    flag_2jl = flag_2jm = false;
  if ( flag_2jl ) countCutflowEvent("2jl_06_met/ht>15");  
  if ( flag_2jm ) countCutflowEvent("2jm_06_met/ht>15");  
  
  if ( flag_2jt && missingET->P4().Et()/sqrt(ht) < 20. )
    flag_2jt = false;  
  if ( flag_2jt ) countCutflowEvent("2jt_06_met/ht>20");  

  if ( !(flag_2jl || flag_2jm || flag_2jt || flag_4jt || flag_5j || flag_6jm || flag_6jt) ) return;    
  
// for 4-6 jets SRs check aplanarity using signal jets
  std::vector<TLorentzVector> jets_mom;
  for (int i = 0; i < jets.size() && jets[i]->PT > 50.; i++ ) 
    jets_mom.push_back(jets[i]->P4());
  
  if ( aplanarity(jets_mom) < 0.04)  flag_4jt = flag_5j = flag_6jm = flag_6jt = false;
    
  if ( flag_4jt ) countCutflowEvent("4jt_06_apl>0.04");     
  if ( flag_5j ) countCutflowEvent("5j_06_apl>0.04");     
  if ( flag_6jm ) countCutflowEvent("6jm_06_apl>0.04");     
  if ( flag_6jt ) countCutflowEvent("6jt_06_apl>0.04");     
  
  if ( !(flag_2jl || flag_2jm || flag_2jt || flag_4jt || flag_5j || flag_6jm || flag_6jt) ) return;      
  
// special m_eff cut depending only on the number of signal jets in a given SR
  
  double meff_nj = missingET->P4().Et() + jets[0]->PT + jets[1]->PT;
  
  if ( flag_4jt ) {
    meff_nj += jets[2]->PT + jets[3]->PT; 
    if ( missingET->P4().Et()/meff_nj < 0.2 ) flag_4jt = false;
  }
  if ( flag_4jt ) countCutflowEvent("4jt_07_met/meff>0.2");
  
  if ( flag_5j) {
    meff_nj += jets[4]->PT; 
    if ( missingET->P4().Et()/meff_nj  < 0.25 ) flag_5j = false;
  }
  if ( flag_5j ) countCutflowEvent("5j_07_met/meff>0.25");  
  
  if ( flag_6jm || flag_6jt) {
    meff_nj += jets[5]->PT; 
    if ( missingET->P4().Et()/meff_nj  < 0.2 ) flag_6jm = flag_6jt = false;
  }
  if ( flag_6jm ) countCutflowEvent("6jm_07_met/meff>0.2");    
  if ( flag_6jt ) countCutflowEvent("6jt_07_met/meff>0.2");   
  
  if ( !(flag_2jl || flag_2jm || flag_2jt || flag_4jt || flag_5j || flag_6jm || flag_6jt) ) return;     
  
// final m_eff cut using all jets Pt > 50  
  
  double meff = missingET->P4().Et();
  for (int i = 0; i < jets.size() && jets[i]->PT > 50.; i++)
    meff += jets[i]->PT;
  
  if ( flag_2jl && meff < 1200. ) flag_2jl = false;
  if ( (flag_2jm || flag_5j || flag_6jm) && meff < 1600. ) flag_2jm = flag_5j = flag_6jm = false;
  if ( (flag_2jt || flag_6jt) && meff < 2000. ) flag_2jt = flag_6jt= false;
  if ( flag_4jt && meff < 2200. ) flag_4jt = false;
  
  if (flag_2jl) {
    countCutflowEvent("2jl_07_meff>1200");
    countSignalEvent("SR2jl");
  }
  
  if (flag_2jm) {
    countCutflowEvent("2jm_07_meff>1600");
    countSignalEvent("SR2jm");
  }
  
  if (flag_2jt) {
    countCutflowEvent("2jt_07_meff>2000");
    countSignalEvent("SR2jt");
  }  
  
  if (flag_4jt) {
    countCutflowEvent("4jt_08_meff>2200");
    countSignalEvent("SR4jt");
  }   
  
  if (flag_5j) {
    countCutflowEvent("5j_08_meff>1600");
    countSignalEvent("SR5j");
  }     
  
  if (flag_6jm) {
    countCutflowEvent("6jm_08_meff>1600");
    countSignalEvent("SR6jm");
  } 
  
  if (flag_6jt) {
    countCutflowEvent("6jt_08_meff>2000");
    countSignalEvent("SR6jt");
  }   
}

void Atlas_conf_2015_062::finalize() {
  // Whatever should be done after the run goes here
}       

double Atlas_conf_2015_062::aplanarity(std::vector<TLorentzVector>& input_jets) {
  
  double mag = 0.;
  for (int k = 0; k < input_jets.size(); k++)
    mag += pow(input_jets[k].Rho(),2);
  
  TMatrixD st(TMatrixD::kZero, TMatrixD(3,3) );
  for (int k = 0; k < input_jets.size(); k++) {
    st(0,0) += input_jets[k].X()*input_jets[k].X()/mag;
    st(0,1) += input_jets[k].X()*input_jets[k].Y()/mag;
    st(0,2) += input_jets[k].X()*input_jets[k].Z()/mag;
    st(1,1) += input_jets[k].Y()*input_jets[k].Y()/mag;
    st(1,2) += input_jets[k].Y()*input_jets[k].Z()/mag;
    st(2,2) += input_jets[k].Z()*input_jets[k].Z()/mag;
  }
  st(1,0) = st(0,1);
  st(2,0) = st(0,2);
  st(2,1) = st(1,2);
  
  TMatrixDEigen eigen(st);
  TMatrixD diag = eigen.GetEigenValues();
  
  std::vector<double> lambdas;
  lambdas.push_back( diag(0,0) );
  lambdas.push_back( diag(1,1) );
  lambdas.push_back( diag(2,2) );
  std::sort (lambdas.begin(), lambdas.end());
  
  return 1.5*lambdas[0];
}       
